@@ -71,7 +71,7 @@ def rospackexec(args):
 
 # for comparing against 'ground truth'
 def rospack_list():
-    return [s.split()[0] for s in rospackexec(['list']).split('\n')]
+    return [s.strip() for s in rospackexec(['list-names']).split('\n') if s.strip()]
 def rospack_find(package):
     return rospackexec(['find', package]).strip()
 def rospack_depends(package):
@@ -107,11 +107,31 @@ def test_RosPack_list():
 def get_package_test_path():
     return os.path.abspath(os.path.join(os.path.dirname(__file__), 'package_tests'))
 
+def test_RosPack_no_env():
+    # regression test for #3680
+    from rospkg import RosPack, ResourceNotFound
+    try:
+        environ_copy = os.environ.copy()
+        if 'ROS_ROOT' in os.environ:
+            del os.environ['ROS_ROOT']
+        if 'ROS_PACKAGE_PATH' in os.environ:
+            del os.environ['ROS_PACKAGE_PATH']
+        r = RosPack()
+        try:
+            r.get_depends('roscpp')
+            assert False, "should have raised"
+        except ResourceNotFound:
+            pass
+    finally:
+        os.environ = environ_copy
+    
+    
 def test_RosPack_get_path():
     from rospkg import RosPack, ResourceNotFound, get_ros_root
 
     path = get_package_test_path()
     foo_path = os.path.join(path, 'p1', 'foo')
+    foo_path_alt = os.path.join(path, 'p2', 'foo')
     bar_path = os.path.join(path, 'p1', 'bar')
     baz_path = os.path.join(path, 'p2', 'baz')
     
@@ -119,7 +139,8 @@ def test_RosPack_get_path():
     print("ROS_ROOT: %s"%(path))
     print("ROS_PACKAGE_PATH: ")
     r = RosPack(ros_root=path, ros_package_path='')
-    assert foo_path == r.get_path('foo')
+    # precedence in this case is undefined as there are two 'foo's in the same path
+    assert r.get_path('foo') in [foo_path, foo_path_alt]
     assert bar_path == r.get_path('bar')
     assert baz_path == r.get_path('baz')
     try:
